/************** BEST VIEWED IN A CODE EDITOR 80 COLUMNS WIDE *******************
*
* ToughBuilt ClipTech Mount
* Benjamen Johnson <workshop.electronsmith.com>
* 20201108
*
* openSCAD Version: 2019.05 
* Licesnse: Creative Commons - Attribution - Share Alike
*           https://creativecommons.org/licenses/by-sa/4.0/
*******************************************************************************/
/*[Hidden]*/
version = "V1.0";

/*[Choose Clip]*/
// Choose the type of clip
type = 0; //[0:Wall Mount,1:Locking Foot,2:Rail Clip]

/*[Clip Parameters]*/
/*******************************************************************************
*        
*         |- top-| 
*          ------      ---
*        /        \     |
*       /          \    trap_height
*      /            \   |
*     /              \  |
*     ---------------- ---
*    |---- bottom ----|
*
*         
*          ------     --- top_gap
*        /  ----  \   ---  
*       /  |    |  \
*      /   |    |   \
*     /    |    |    \
*     ----------------
*          |    |
*        base_width
*
*******************************************************************************/
// length of the base of the trapezoid part of the clip
bottom = 46.4;

// length of the top of the trapezoid part of the clip
top = 24;

// height of the trapezoid part of the clip
trap_height = 50.4;

// Width of the base part
base_width = 21.5;

// Depth of the base
base_depth = 4.2;

// Add extra depth to make the clip stand out
extra_depth = 11;

// Space at top where base doesn't extend
top_gap = 3.75;

// how far down fom the top of the clip is the latch indent
latch_drop = 8.25;

// How wide is the latch indent
latch_width = 12.5;

// how deep into the clip does the latch indent extend
latch_depth = 2.5;

/*[Hidden]*/   
scale_height = 42;

trapezoid_thickness = 2.5;
chamfered_thickness = 1.75;

scale_fac = scale_height/trap_height;

base_height = trap_height - top_gap;

/*[Hidden]*/
// distance to jump over from axis to start top 
jump = (bottom-top)/2;

//define the points for trapezoid part of the clip
TRAPEZOID = [[0,0],[bottom,0],[jump+top,trap_height],[jump, trap_height]];

// matrix to move the trapezoid to the center;
CENTER =   [[-bottom/2,-trap_height/2],
            [-bottom/2,-trap_height/2],
            [-bottom/2,-trap_height/2],
            [-bottom/2,-trap_height/2]];

// center the trapezoid
CENTERED_TRAPEZOID = TRAPEZOID+CENTER;

/*[Rail Parameters]*/
/*******************************************************************************
*             short
*         |-- _width--|
*         |           |    
*          -----------    ---
*    --- /             \   |
*short_ |               |  rail_depth 
* depth |               |  |
*    --- ---------------  ---
*       |    rail_      |
*       |--- width ---- |
*
*******************************************************************************/
// Width of Packout rail
rail_width = 29.5;

// Depth of Packout rail
rail_depth = 13.5;

// Width of inside part of rail
short_width = 16;

// Depth of packout rail before angle cut
short_depth = 7;

// Thickness of clamp walls
wall_thickness = 11;

// Height of clamp
rail_clamp_height = 25;

// What is the wrench size of the hex nut to be pressed
hex_nut_size = 8.62;

// Adjust the pressed hex nut fit
hex_nut_fit = 1.01;

// How deep should the hex nut sit in the strut nut?
hex_nut_depth = 4;

gap = (rail_width-short_width)/2;

RAIL_EXTRUSION= [[0,0],
                [rail_width,0],
                [rail_width,short_depth],
                [short_width+gap,rail_depth],
                [gap,rail_depth],
                [0,short_depth]];

RAIL_CENTER=[[-rail_width/2,-rail_depth/2],
            [-rail_width/2,-rail_depth/2],
            [-rail_width/2,-rail_depth/2],
            [-rail_width/2,-rail_depth/2],
            [-rail_width/2,-rail_depth/2],
            [-rail_width/2,-rail_depth/2]];

RAIL_PROFILE = RAIL_EXTRUSION+RAIL_CENTER;

/*[Screw Parameters]*/
/*******************************************************************************
*
*      |----------|<-- counter_diameter
* ----  ---------- ---
*   |   \        /  |
*   |    \      /   | <--counter_depth
* total_  |    |   ---       
* depth   |    |
*   |     |    |
*   |     |    |
* -----    ----
*         |----|<-- hole_dia
*
*******************************************************************************/
// Do you want a counterbore or countersink?
hole_type = 0; //[0:Counterbore, 1: Countersink]

// Diameter of the holes
hole_dia = 5;

// If you are using countersinks, what angle?
countersink_angle = 82; // [60,82,90,100,110,120]

// Depth of the countersink or bore
counter_depth = 4;

// Counterbore diameter (countersink uses depth and angle)
counter_dia = 9;

/*[Connector Parameters]*/
/*******************************************************************************
* Locking Packout Foot Connector Definition
*     -------
*    (   O   )
*     -------
*   |         |
*   |---------| gear_hole_diameter 
*******************************************************************************/
// How tall is the connector on top of the shaft
connector_height = 2.1;//[1:0.1:4]

// Diameter of the gear shaft (defines size of connector)
gear_hole_dia = 22;

/*[Stamp Options]*/
/*******************************************************************************
* Stamp Options
*******************************************************************************/
// Which font do you want to use
font = "REGISTRATION PLATE UK:style=Bold";

// How big you want the text
font_size = 6;//[0:0.5:8]

// How thick the font is (back to front)
font_thickness = 0.6;

/*[Hidden]*/
/*******************************************************************************
* Calculations
*******************************************************************************/
// enum type 
WALL_MOUNT=0; LOCKING_FOOT=1; RAIL_CLIP=2;

// Calculate the point to point diameter of the pressed nut
pt_to_pt_hex_nut_dia = 2*(hex_nut_size/2)/cos(30)* hex_nut_fit;

// Calculate the diameter of the countersink hole
bottom_thickness = 3;
countersink_dia = 2*tan(countersink_angle/2)*bottom_thickness;

// add a tiny amount to cuts to make them render properly
$dl=0.01;

face_pos = trapezoid_thickness+chamfered_thickness+base_depth/2;

/*******************************************************************************
* Make it
*******************************************************************************/
if(type == LOCKING_FOOT)
    translate([0,0,trap_height/2])
        make_locking_foot();

else if(type == WALL_MOUNT)
    translate([0,0,trap_height/2])
        make_wall_mount();

else if(type == RAIL_CLIP){
    translate([0,0,trap_height/2])
        make_rail_clip();
    
    translate([0,-25,0])
        rotate([0,0,180])
            make_rail_clamp();
    }//end else if

/*******************************************************************************
* Make the rail clamp
*******************************************************************************/
module make_rail_clamp(){
    difference(){
        
        // extrude rail clamp
        linear_extrude(height = rail_clamp_height,$fn=100) 
            difference() {
                offset(delta = wall_thickness) 
                    polygon(RAIL_PROFILE);
                polygon(RAIL_PROFILE);
            }//end difference
        
        // Cut screw holes
        translate([-(rail_width+wall_thickness)/2,0,rail_clamp_height/2])
        rotate([90,0,0])
        cylinder(d=5,h=rail_depth+2*wall_thickness+$dl,center = true,$fn = 50);
            
        translate([(rail_width+wall_thickness)/2,0,rail_clamp_height/2])
        rotate([90,0,0])
        cylinder(d=5,h=rail_depth+2*wall_thickness+$dl,center = true,$fn = 50);
          
        // cut nut holes
        translate([-(rail_width+wall_thickness)/2,rail_depth/2,rail_clamp_height/2])
        rotate([90,30,0])
        cylinder(d=pt_to_pt_hex_nut_dia,h=rail_depth,center = true,$fn = 6);
            
        translate([(rail_width+wall_thickness)/2,rail_depth/2,rail_clamp_height/2])
        rotate([90,30,0])
        cylinder(d=pt_to_pt_hex_nut_dia,h=rail_depth,center = true,$fn = 6);
    
    // cut into two pieces
    translate([0,-rail_depth/2,rail_clamp_height/2])
    cube([rail_width+2*wall_thickness+$dl,1,rail_clamp_height+$dl],center = true);
    
    translate([0,-rail_depth/2-6,rail_clamp_height/2])
    cube([rail_width+2*wall_thickness+$dl,12,rail_clamp_height+$dl],center = true);

    } // end difference
} //end module
/*******************************************************************************
* Module for making the locking foot version of the clip
*******************************************************************************/
module make_rail_clip(){
    union(){  
        difference(){
            make_clip();
            
            // Stanp version
            translate([0,face_pos-font_thickness,-12])
                rotate([90,0,180])
                    stamp_version(version, font_size, font_thickness);
        } //end difference
             
        translate([0,-extra_depth/2-base_depth/2,(base_height-rail_clamp_height-top_gap)/2])
        difference(){
            // make pentagon shape
            h=trap_height-top_gap;
            stub_height = h-rail_clamp_height;
            hull(){
                cube([51.5,extra_depth,rail_clamp_height],center = true);
                
                translate([0,0,-rail_clamp_height/2-stub_height/2])
                cube([base_width,extra_depth,stub_height],center = true);
            } //end hull
            
            // make screw holes (countersink to 9mm)
            screw_offset = (rail_width + wall_thickness)/2;
            translate([-screw_offset,5,0])
            rotate([-90,0,0])
            screw_hole();
            
            translate([screw_offset,5,0])
            rotate([-90,0,0])
            screw_hole();
            
        } //end difference
    }//end union
} // end module

/*******************************************************************************
* Module for making the locking foot version of the clip
*******************************************************************************/
module make_locking_foot(){
    difference(){
        make_clip();
        
        // make the connector
        translate([0,-$dl-extra_depth,-2])
            rotate([90,0,0])
                connector();

        translate([0,face_pos,-2])
            rotate([-90,0,0])
                screw_hole();
        
        // Stamp version
        translate([0,face_pos-font_thickness,-12])
        rotate([90,0,180])
        stamp_version(version, font_size, font_thickness);

    } //end difference
} //end module
    
/*******************************************************************************
* Module for making the wall mount version of the clip
*******************************************************************************/
module make_wall_mount(){
    difference(){
        make_clip();
        
        // drill holes
        translate([0,face_pos-latch_depth,-14.4])
            rotate([-90,0,0])
                screw_hole();

        translate([0,face_pos-latch_depth,11])
            rotate([-90,0,0])
                screw_hole();
        
        // Stamp version
        translate([0,face_pos-font_thickness,-3])
            rotate([90,0,180])
                stamp_version(version, font_size, font_thickness);

    } //end difference
} //end module
/*******************************************************************************
* Module for making the clip
*******************************************************************************/
module make_clip(){
    
ZERO_LATCH_POS = [0,face_pos-latch_depth/2+$dl,(trap_height-latch_width)/2];

    difference(){
        union(){
             //make chamfered trapezoid top
            translate([0,base_depth/2+trapezoid_thickness,0])
            rotate([90,0,180])
            linear_extrude(height = chamfered_thickness,scale = scale_fac) 
                polygon(CENTERED_TRAPEZOID);
            
            //make trapezoid body
            translate([0,base_depth/2,0])
            rotate([90,0,180])
            linear_extrude(height = trapezoid_thickness) 
                polygon(CENTERED_TRAPEZOID);
            
            //make base
            translate([0,0,-top_gap/2])
            cube([base_width,base_depth,base_height],center=true);
            
            //extend base
            if(type != RAIL_CLIP) //PrusaSlicer won't slice with this overlap
                translate([0,-(base_depth+extra_depth)/2,-top_gap/2])
                    cube([base_width,extra_depth,base_height],center=true);
            
        } //end union
        
        // make square latch hole
        translate(ZERO_LATCH_POS - [0,0,latch_drop])
        cube([latch_width,latch_depth,latch_width],center=true);
        
    } //end difference
} //end module

/*******************************************************************************
* Module for drilling counterbored or countersuck screw holes
*******************************************************************************/
module screw_hole(){
// Stick above the plane to get a good cut in uneven surfaces    
extra_height = 3;
    
// Calculate the diameter of the countersink hole
countersink_dia = 2*tan(countersink_angle/2)*(counter_depth+extra_height);

    union(){
        translate([0,0,-10])cylinder(d=hole_dia,h=20,center=true,$fn=25);
        if(hole_type == 0)
            translate([0,0,-counter_depth/2+extra_height/2])
            cylinder(d=counter_dia,h=counter_depth+extra_height,center=true,$fn=25);
        else
            translate([0,0,-counter_depth/2+extra_height/2])
            cylinder(d2=countersink_dia+hole_dia,d1=hole_dia,h=counter_depth+extra_height,center=true,$fn=25);
    }// end union
} // end module

/*******************************************************************************
* Module to create the connector hole for the locking packout foot
*******************************************************************************/
module connector(){

CUTOFF_DEF = [20,10,connector_height+$dl];
cutoff_pos = 10.1;
    
    translate([0,0,connector_height/2])
    difference(){
        cylinder(d=gear_hole_dia,h=connector_height,center=true,$fn=100);
        translate([0,cutoff_pos,0])
            cube(CUTOFF_DEF,center=true);
        translate([0,-cutoff_pos,0])
            cube(CUTOFF_DEF,center=true);
    } // end difference
} //end module

/*******************************************************************************
* Module for putting the version number on the print
*******************************************************************************/
module stamp_version(version_text,size,thickness){
    
    color("white")
    linear_extrude(height = thickness)
        text(text = version_text, font = font, size = size, valign="center",halign = "center");
} // end module
